#!/bin/sh
#shopt -s nocasematch

SYSFS_INPUT_EVENT_DIR=/sys/class/input
CONFIG_ROOT=/usr/share/mxt-config/configs
TOOLS=/usr/bin

# XXX This is removed due to making things as fast as possible!
#find_event_device () {
#    DEVICE_NAME="Atmel mXT Touchscreen"

#    cd $SYSFS_INPUT_EVENT_DIR;
#    for i in event* ; do
#	devname=`cat $i/device/name`
#	#echo ${i}: ${devname}
#	if [ "${devname}" = "${DEVICE_NAME}" ] ; then
#	    echo ${i};
#	    return 0;
#	fi
#   done
#
#    return 1;
#}


construct_device_path () {

# XXX Here we take a shortcut for speed!
    echo "/sys/class/i2c-adapter/i2c-2/2-004b"
    return 0;

    device_dir=${SYSFS_INPUT_EVENT_DIR}/$1/device/device;
    if [ -d $device_dir ] ; then
	echo $device_dir
	return 0;
    fi

    return 2;
}

sysinfo_get_product () {
    # XXX HACK We need to pad the digit part to three digits!
    pr=`sysinfoclient -g /component/product | sed -e 's/.*= //' | tr '[A-Z]' '[a-z]' | sed -e 's/\(..\)-\([0-9][0-9]_\)/\1-0\2/`

    if [ $pr ] ; then
	echo $pr;
	return 0;
    fi

    return 3;
}

sysinfo_get_hwid() {
    hid=`sysinfoclient -g /component/hw-build | sed -e 's/.*= //'`

    if [ $hid ] ; then
	echo $hid;
	return 0;
    fi

    return 3;
}

display_get_hwid() {
    did=`cat /sys/devices/omapdss/display0/hw_revision | tr '[A-Z]' '[a-z]'`
    if [ $did ] ; then
	echo $did;
	return 0;
    fi
    return 4;
}

find_config_by_id() {
    match=$1
    filename=${CONFIG_ROOT}/id_to_file_map.txt

    cfg=`mxt_id2cfg ${filename} ${match}`
    result=$?
    if [ $result -ne 0 ] ; then
	echo "Config file not resulved";
	return $result;
    fi

    echo ${cfg};

    return 0;
}

config_get_base_name() {
    dispid=`echo ${DISPLAY_HWID} | tr -d '.'`

    search_string=${PRODUCT}_${PRODUCT_HWID}_${dispid}_${FIRMWARE_VERSION}${FIRMWARE_REVISION}
    #find_config_by_id $search_string
    fname=`find_config_by_id $search_string`
    result=$?
    if [ $result -ne 0 ] ; then
	echo "Could not find config for '${search_string}'";
	return $result;
    fi
    echo ${CONFIG_ROOT}/$fname;
    return 0;
}

config_get_crc() {
    crc=`${TOOLS}/mxt_crcbin ${1}`
    result=$?
    if [ $result -ne 0 ] ; then
	echo "Could not calculate crc for '${1}', returned ${crc}";
	return $result;
    fi
    echo $crc;
    return 0;
}

get_machine_information() {
#    echo "0** (`cut -d " " -f 1 /proc/uptime`)"

#    evdev=`find_event_device`;
#    result=$?
#    if [ $result -ne 0 ]; then
#	echo "Did not find correct event device!";
#	return $result;
#    fi

#   echo "1** (`cut -d " " -f 1 /proc/uptime`)"

    DEVICE_PATH=`construct_device_path $evdev`
    result=$?
    if [ $result -ne 0 ]; then
	echo "Did not find correct device path!";
	return $result;
    fi

#    echo "2** (`cut -d " " -f 1 /proc/uptime`)"

    PRODUCT=`sysinfo_get_product`
    PRODUCT_HWID=`sysinfo_get_hwid`

    TS_DISABLED=`cat ${DEVICE_PATH}/disable_ts`;
    result=$?
    if [ $result -ne 0 ]; then
	echo "Did not find disable sysfs node";
	return $result;
    fi

    WAIT_MODE=`cat ${DEVICE_PATH}/wait_for_gesture`;
    result=$?
    if [ $result -ne 0 ]; then
	echo "Did not find wait sysfs node";
	WAIT_MODE="0x00";
    fi

    if [ "${TS_DISABLED}" != "0" ] ; then
	echo "Touchscreen is not on, setting on";
	echo 0 >${DEVICE_PATH}/disable_ts;
	sleep 0.3
    fi

    if [ "${WAIT_MODE}" != "0x00" ] ; then
	echo "Wait mode was on, disabling";
	echo 0 >${DEVICE_PATH}/wait_for_gesture;
	sleep 0.3
    fi

    FIRMWARE_STR=`cat ${DEVICE_PATH}/firmware_version`
    result=$?
    if [ $result -ne 0 ]; then
	echo "Did not find firmware version!";
	return $result;
    fi

#    echo "3** (`cut -d " " -f 1 /proc/uptime`)"

    FIRMWARE_VERSION=`echo ${FIRMWARE_STR} | cut -d "." -f 1`
    FIRMWARE_REVISION=`echo ${FIRMWARE_STR} | cut -d "." -f 2`

    NV_CRC=`cat ${DEVICE_PATH}/nv_config_crc`
    result=$?
    if [ $result -ne 0 ]; then
	echo "Did not find nv_config_crc!";
	return $result;
    fi

#    echo "4** (`cut -d " " -f 1 /proc/uptime`)"

    RUNNING_CRC=`cat ${DEVICE_PATH}/config_crc`
    result=$?
    if [ $result -ne 0 ]; then
	echo "Did not find running config_crc!";
	return $result;
    fi

#    echo "5** (`cut -d " " -f 1 /proc/uptime`)"

    DISPLAY_HWID=`display_get_hwid`
    result=$?
    if [ $result -ne 0 ]; then
	echo "Did not find display hw!";
	return $result;
    fi

#    echo "6** (`cut -d " " -f 1 /proc/uptime`)"

    CFG_BASE_NAME=`config_get_base_name`
    result=$?
    if [ $result -ne 0 ] ; then
	echo "Can't find a basename for this combination";
	echo ${CFG_BASE_NAME};
	return $result;
    fi

#    echo "7** (`cut -d " " -f 1 /proc/uptime`)"

    CFG_BIN_FILE=${CFG_BASE_NAME}.bin

    CFG_CRC=`config_get_crc ${CFG_BIN_FILE}`
    result=$?
    if [ $result -ne 0 ]; then
	echo "Did not find config crc";
	echo ${CFG_CRC};
	return $result;
    fi
#    echo "9** (`cut -d " " -f 1 /proc/uptime`)"
}

reset_controller() {
    echo 1 >${DEVICE_PATH}/disable_ts
    sleep 0.2
    echo 0 >${DEVICE_PATH}/disable_ts
}

update_running_config() {
    cp ${CFG_BIN_FILE} /lib/firmware
    cfg_file_base=`basename ${CFG_BIN_FILE}`
    if [ -r /lib/firmware/$cfg_file_base ]; then
	echo "Updating running configuration from ${RUNNING_CRC} to ${CFG_CRC}";
	echo 1 >${DEVICE_PATH}/clear_config
	echo $cfg_file_base >${DEVICE_PATH}/write_config
	RUNNING_CRC=`cat ${DEVICE_PATH}/config_crc`
	echo "Running now ${RUNNING_CRC}";
	if [ "${CFG_CRC}" == "${RUNNING_CRC}" ] ; then
	    echo "Update running config successful, resetting";
	    reset_controller;
	    return 0;
	else
	    echo "Update failed";
	    return 102;
	fi

    else
	echo "Can't find config in /lib/firmware, bailing out";
	return 101;
    fi
}

backup_config() {
    echo 1 >${DEVICE_PATH}/backup_config
    count=0
    NV_CRC=0

    while [[ ${count} -lt 10 ]] ; do
	if [ "${NV_CRC}" == "${CFG_CRC}" ] ; then
	    return 0;
	fi
#	echo $count;
	count=$((${count}+1));
	sleep 0.05
#	echo "count ${count}"
	NV_CRC=`cat ${DEVICE_PATH}/nv_config_crc`
    done

    if [ "${NV_CRC}" == "${CFG_CRC}" ] ; then
	return 0;
    fi

    return 205;
}

update_config_once() {
    if [ "${RUNNING_CRC}" != "${CFG_CRC}" ] ; then
	update_running_config
	result=$?
	if [ $result -ne 0 ] ; then
	    echo "Update failed with code $result";
	    return $result;
	fi
    else
 	echo "Running config already same as ${CFG_CRC}";
    fi

    if [ "${NV_CRC}" == "${CFG_CRC}" ] ; then
	echo "Configuration already in nv memory";
	return 0;
    fi

    echo "Nonvolatile crc ${NV_CRC} differs from running ${RUNNING_CRC}, updating";

    # XXX Make driver atomic on these!
    backup_config
    result=$?

    if [ $result -ne 0 ] ; then
	echo "Backup failed";
    else
	echo "Update successful. Performing controller reset"
	reset_controller;
    fi

    return $result;
}

get_machine_information
result=$?
if [ $result -ne 0 ] ; then
    echo "Failed to gather all information from environment!";
    exit $result;
fi

echo "Product  : ${PRODUCT}"
echo "Device   : ${DEVICE_PATH}"
echo "FW Ver   : ${FIRMWARE_VERSION}.${FIRMWARE_REVISION}"
echo "NV Cfg   : '${NV_CRC}'"
echo "Running  : '${RUNNING_CRC}'"
echo "Disp hw  : ${DISPLAY_HWID}"
echo "CFG      : ${CFG_BIN_FILE}"
echo "CFG_CRC  : '${CFG_CRC}'"
echo "WaitMode : '${WAIT_MODE}'"
echo "Disabled : '${TS_DISABLED}'"

# echo "u1* (`cut -d " " -f 1 /proc/uptime`)"

update_config()
{
    count=0

    while [[ ${count} -lt 10 ]] ; do
	update_config_once
	result=$?
	if [ $result -eq 0 ] ; then
	    return 0;
	fi
	count=$((${count}+1));
	sleep 0.2
	echo "count ${count}"
    done

    return $result;
}

update_config
result=$?

#echo "u2* (`cut -d " " -f 1 /proc/uptime`)"

CURRENT_TS_DIS=`cat ${DEVICE_PATH}/disable_ts`;
if [ "${CURRENT_TS_DIS}" != "0" ] ; then
    echo "WARNING: Someone turned touchscreen off during configuration!"
fi

if [ "${TS_DISABLED}" != "0" ] ; then
    sleep 0.2;
    echo "Restoring ts power state";
    echo 1 >${DEVICE_PATH}/disable_ts;
fi

if [ "${WAIT_MODE}" != "0x00" ] ; then
    sleep 0.2;
    echo "Restoring wait mode state";
    echo ${WAIT_MODE} >${DEVICE_PATH}/wait_for_gesture;
fi

if [ $result -ne 0 ] ; then
    echo "Update returned $result";
    exit $result
fi

exit 0
